package com.yaoyicloud.service.impl;

import static com.yaoyicloud.config.SessionInterceptor.SESSION_MAP;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.util.ArrayList;
import java.util.List;
import com.yaoyicloud.render.cso.EntAppendixRender;
import com.yaoyicloud.render.cso.EntPromotionDetailsRender;
import org.apache.poi.xwpf.usermodel.XWPFDocument;
import org.springframework.stereotype.Service;
import com.yaoyicloud.config.CommonDataCache;
import com.yaoyicloud.config.FilerepoProperties;
import com.yaoyicloud.config.ReportPathManager;
import com.yaoyicloud.constant.enums.ModuleType;
import com.yaoyicloud.entity.ReportGenerationResult;
import com.yaoyicloud.render.cso.EntHeaderSectionRender;
import com.yaoyicloud.render.cso.EntPromotionSummaryRender;
import com.yaoyicloud.service.CsoReportService;
import com.yaoyicloud.tools.DocxUtil;

import cn.hutool.core.util.IdUtil;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;

@Slf4j
@Service
@RequiredArgsConstructor
public class CsoReportServiceImpl implements CsoReportService {
    private final FilerepoProperties filerepoProperties;
    private final ReportPathManager reportPathManager;
    private final CommonDataCache commonDataCache;

    /**
     * 处理单个模块的报告生成
     */
    @SuppressWarnings("checkstyle:ParameterNumber")
    private String processModule(ModuleType moduleType, String data,
        byte[] templateBytes, String sessionId, Long relationId) throws Exception {
        String reportPath;

        switch (moduleType) {
            case CSO_HEADER:
                reportPath = new EntHeaderSectionRender(sessionId, filerepoProperties, commonDataCache)
                    .renderDocx(data, templateBytes, String.valueOf(relationId));
                break;
            case CSO_PROMOTION_SUMMARY:
                reportPath = new EntPromotionSummaryRender(sessionId, filerepoProperties, commonDataCache)
                    .renderDocx(data, templateBytes, String.valueOf(relationId));
                break;
            case CSO_PROMOTION_DETAILS:
                reportPath = new EntPromotionDetailsRender(sessionId, filerepoProperties, commonDataCache)
                    .renderDocx(data, templateBytes, String.valueOf(relationId));
                break;
            case CSO_APPENDIX:
                reportPath = new EntAppendixRender(sessionId, filerepoProperties, commonDataCache)
                    .renderDocx(data, templateBytes, String.valueOf(relationId));
                break;
            default:
                throw new UnsupportedOperationException("Unsupported module type: " + moduleType);
        }
        return reportPath;
    }

    @Override
    public ReportGenerationResult createCsoCheckReport(String data, byte[] templateBytes, Long relationId,
        ModuleType moduleType) throws Exception {
        // 1. 将字节流模版写入临时文件
        File tempTemplateFile = File.createTempFile("template_", ".docx");
        try (FileOutputStream fos = new FileOutputStream(tempTemplateFile)) {
            fos.write(templateBytes);
        }

        String sessionId = SESSION_MAP.get(relationId.toString());

        String reportPath = processModule(moduleType, data, templateBytes, sessionId, relationId);
        // 处理指定模块
        reportPathManager.addReportPath(sessionId, reportPath);

        String mergedReportPath = null;
        boolean isLastModule = reportPath.contains("Appendix");

        if (isLastModule) {
            List<String> reportPaths = reportPathManager.removeSession(sessionId);
            XWPFDocument targetDoc;
            try (FileInputStream fis = new FileInputStream(reportPaths.get(0))) {
                targetDoc = new XWPFDocument(fis);
            }
            List<XWPFDocument> sourceDocs = new ArrayList<>();
            for (int i = 1; i < reportPaths.size(); i++) {
                try (FileInputStream fis = new FileInputStream(reportPaths.get(i))) {
                    sourceDocs.add(new XWPFDocument(fis));
                }
            }
            String label = relationId + "_" + IdUtil.fastSimpleUUID();
            mergedReportPath = filerepoProperties.getBasePath() + "/" + label + ".docx";
            // 调用合并方法
            DocxUtil.mergeDocx(targetDoc, sourceDocs, mergedReportPath, relationId);
        }

        return new ReportGenerationResult(reportPath, mergedReportPath, isLastModule);
    }
}
